 /****************************************************************/
 /*      MACRO PARA LA DETECCION DE OBSERVACION ATIPICAS         */
 /*                                                              */
 /*    NAME: OUTLIERS_MULT                                       */
 /*   TITLE: Detecc, de datos atipicos multivariantes            */
 /* PRODUCT: IML BASE STAT                                       */
 /*  SYSTEM: ALL                                                 */
 /*    KEYS: DEPURACION DE DATOS                                 */
 /*   PROCS:                                                     */
 /*    DATA:                                                     */
 /*                                                              */
 /* SUPPORT: saswfk                      UPDATE:  O1OCT05        */
 /****************************************************************/

 /*--------------------------------------------------------------------
Esta macro detecta las observaciones atpicas multivariantes y
univariantes. Los parmetros de entrada para la macro consisten en el
conjunto de datos que contiene la informacin y las variables que van a
ser examinadas de forma conjunta y la variable que puede identificar a 
las observaciones (ID=). 
Las salidas que produce la macro quedan guardadas en los siguientes
conjuntos de datos: 
*	ESTANDARIZADO   Este conjunto de datos recoge los valores
					estandarizados (tipificados media=0, varianza=1)
					para todas las variables y se indica en base a ellos
                    cuales son las observaciones con problemas segn 
					tres categoras: sin problemas (valor estand<3,
					con problemas (valor estand entre 3 y 3.5) y 
					claramente atpicos (estand > 3.5) para alguna variable. 
					Es decir es una deteccin univariante de outliers.

*	OUTLIERS. Este conjunto contiene las distancias al punto central,
				as como el p-valor asociado si las citadas distancias 
				provienen de una ley chi-cuadrado as como el valor de 
				referencia que no deben superar para poder considerarse
			    observaciones homogeneas y la conclusin o veredicto que
				se toma frente a las citadas observaciones. Todas ellas 
			    vienen ordenadas por el rango de las distancias, adems 
                se incluye el nmero de observacin con el objeto de que
                sea perfectamente detectada. 

Los parmetros a incluir son exclusivamente: 
"	Data= c.datos
"	Var= lista de variables (sin abreviaciones) que van a ser estudiadas. 

**************************************************************************/
	
DATA BARCOS;
  INPUT X1-X4;
cards;
  1889  1651  1561  1778
  2403  2048  2087  2197
  2119  1700  1815  2222
  1645  1627  1110  1533
  1976  1916  1614  1883
  1712  1712  1439  1546
  1943  1685  1271  1671
  2104  1820  1717  1874
  2983  2794  2412  2581
  1745  1600  1384  1508
  1710  1591  1518  1667
  2046  1907  1627  1898
  1840  1841  1595  1741
  1867  1685  1493  1678
  1859  1649  1389  1714
  1954  2149  1180  1281
  1325  1170  1002  1176
  1419  1371  1252  1308
  1828  1634  1602  1755
  1725  1594  1313  1646
  2276  2189  1547  2111
  1899  1614  1422  1477
  1633  1513  1290  1516
  2061  1867  1646  2037
  1856  1493  1356  1533
  1727  1412  1238  1469
  2168  1896  1701  1834
  1655  1675  1414  1597
  2326  2301  2065  2234
  1490  1382  1214  1284
;

%MACRO outliers_MULT(DATA=, VAR= ,id=); 
proc corr data=&DATA cov outp=DATA_2;
VAR &VAR; RUN; 


proc iml;
  USE &DATA;
  read all var {&VAR}into X;
  use DATA_2;
    read all where(_Type_='COV') VAR {&VAR} INTO S;
    READ all WHERE (_TYPE_='MEAN') VAR {&VAR} INTO MEDIAT  ;
    sinver=inv(S);

    n=nrow(x);
    vecuno=j(n,1,1);  /** VECTOR Nx1 DE 1**/

    medias=vecuno*(mediat);    /** CREA MATRIZ NxP CUYAS COLUMNAS SON LAS P MEDIAS**/
    xmedia=x-medias;
    distanci=VECDIAG(xmedia*sinver*t(xmedia));  /** CREA VECTOR DE DISTANCIAS al cuadrado**/
    RANGOS=RANK(DISTANCI);
	P=NCOL ({&VAR}); 
    

	alfa=1-(0.8**(1/n));
	valor_referente=cinv(1-alfa,p); 
	pvalor=j(n,1,.);
	conclusion=j(n,1,'            '); 
	MAXIMO=j(n,1,.);
	minimo=j(n,1,.);
	llamada=j(n,1,'sin problemas'); 

	estandarizado=xmedia*diag(s##-0.5); 
		do i=1 to n; 
			pvalor[i]=1-probchi(distanci[i], p); 
			if pvalor[i]<alfa then conclusion[i]='caso atpico'; 
			else conclusion[i]='caso lgico'; 
			maximo[i]=ESTANDARIZADO[I,<>];
			minimo[i]=estandarizado[i,><]; 
			if -minimo[i]>maximo[i] then maximo[i]=-minimo[i]; 
			if maximo[i]>3  then llamada[i]='PROBLEMAS'; 
			IF MAXIMO[I]>3.5 then llamada[i]='ATIPICO'; 

		end; 
	observacion=do(1,n,1);

/**creacion de conjuntos de datos**/

  create outliers var {observacion distanci rangos pvalor conclusion}; 
  append;  
	close	outliers; 
  variables={&var};  

	create estandarizado from estandarizado [colname=variables] ; 
  append from estandarizado; 
  close estandarizado; 
	
  create univariante var {maximo llamada}; append;close univariante; 

/***obtencin de salidas**/
  print alfa valor_referente;
  quit; 

%if id=' ' %then %do; 
  proc sort data=outliers; by rangos; proc print data=outliers;run;

	data estandarizado; 
	  merge estandarizado univariante;run;
  proc print data=estandarizado; TITLE 'DATOS ESTANDARIZADOS Y OUTLIERS';
run; 
  TITLE ; 
	%end; 
%else %do; 
	data referencia; set &data; keep &id; run;
	data outliers; merge referencia outliers; run;
	proc sort data=outliers; by rangos; proc print data=outliers;run;

	data estandarizado; 
	  merge estandarizado univariante referencia;run;
  proc print data=estandarizado; TITLE 'DATOS ESTANDARIZADOS Y OUTLIERS';
run; 
  TITLE ; 
  %end;



  %mend outliers_mult; 



  ejemplo: 
%outliers_mult (data=barcos, var=x1 x2 x3 x4, id=pais); 
